import React, { memo, useEffect } from 'react';
import { useDispatch } from 'react-redux';
import { initiateForm, dropForm } from '../action-creators';
import { FormContext } from '../form-context';
import { RootValidator } from '../root-validator';
export const FormProvider = memo(({ children, dropOnUnmount, initialValues = {}, name, validate, }) => {
    const dispatch = useDispatch();
    useEffect(() => {
        if (validate) {
            const rootValidator = new RootValidator(name);
            rootValidator.applyFormValidator(validate);
        }
    }, []);
    useEffect(() => {
        dispatch(initiateForm(name, initialValues));
    }, [initialValues]);
    useEffect(() => () => {
        if (dropOnUnmount) {
            dispatch(dropForm(name));
        }
    }, []);
    return (React.createElement(FormContext.Provider, { value: name }, children));
});
//# sourceMappingURL=form-provider.js.map