import { useCallback, useContext } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { FormContext } from '../form-context';
import { changeValue, changeValueAndValidate, clearValue, validateField, } from '../middleware/middleware-actions';
import { RootValidator } from '../root-validator';
import { createGetFormFieldErrors, createGetFormFieldSafetyValue, createGetIsFormFieldPristine, createGetIsFormFieldValid } from '../selectors';
export const useField = (fieldName, { changeValueGetter, validateOnChange, validators, } = {}) => {
    var _a;
    const formName = useContext(FormContext);
    if (!formName) {
        throw new ReferenceError(`It seems either you try to call useField hook for ${fieldName} field out of any Form
component, or did not set a formName to Form you provide.`);
    }
    if (validators) {
        const rootValidator = new RootValidator(formName);
        rootValidator.applyFieldValidators(fieldName, validators);
    }
    const value = (_a = useSelector(createGetFormFieldSafetyValue(formName, fieldName))) !== null && _a !== void 0 ? _a : null;
    const errors = useSelector(createGetFormFieldErrors(formName, fieldName));
    const isPristine = useSelector(createGetIsFormFieldPristine(formName, fieldName));
    const isFieldValid = useSelector(createGetIsFormFieldValid(formName, fieldName));
    const dispatch = useDispatch();
    const clear = useCallback(() => {
        dispatch(clearValue(formName, fieldName));
    }, [dispatch, fieldName, formName]);
    const onChange = useCallback((inputData) => {
        const change = validateOnChange ? changeValueAndValidate : changeValue;
        const nextValue = changeValueGetter ? changeValueGetter(inputData) : inputData;
        dispatch(change(formName, fieldName, nextValue));
    }, [changeValueGetter, dispatch, fieldName, formName, validateOnChange]);
    const validate = useCallback(() => {
        dispatch(validateField(formName, fieldName));
    }, [dispatch, fieldName, formName]);
    return {
        clear,
        errors,
        isFieldValid,
        isPristine,
        onChange,
        validate,
        value,
    };
};
//# sourceMappingURL=use-field.js.map