import { EEasyFormActionType, EEasyFormFieldStatus, EEasyFormField } from './enums';
import { deepEqual } from './utils';
const setEasyFormValue = (formState, formName, categoryName, fieldName, value) => {
    var _a;
    // @ts-ignore
    const newForm = (_a = formState[formName]) !== null && _a !== void 0 ? _a : {};
    return Object.assign(Object.assign({}, formState), { [formName]: Object.assign(Object.assign({}, newForm), { [categoryName]: Object.assign(Object.assign({}, (newForm[categoryName] ? newForm[categoryName] : {})), { [fieldName]: value }) }) });
};
/*
 * To fill the function generic, use two types
 * 1. enum of formNames, used in the application, i.e.:
 *    enum EFormName { Profile, Contacts }
 * 2. type of each form fields:
 *    type TForm = {
 *      [EFormName.Profile]: {
 *        name: string;
 *        age: number;
 *      };
 *      [EFormName.Contacts]: {
 *        phone: string;
 *        email: string;
 *      };
 *    };
 */
export const easyFormReducer = (state = {}, action) => {
    var _a, _b, _c;
    switch (action.type) {
        case EEasyFormActionType.SetFieldValue: {
            const { formName, fieldName, value } = action.payload;
            return setEasyFormValue(state, formName, EEasyFormField.Values, fieldName, value);
        }
        case EEasyFormActionType.ClearFieldValue: {
            const { formName, fieldName } = action.payload;
            if (!((_a = state[formName]) === null || _a === void 0 ? void 0 : _a[EEasyFormField.Values])) {
                return state;
            }
            return setEasyFormValue(state, formName, EEasyFormField.Values, fieldName, null);
        }
        case EEasyFormActionType.SetFieldStatus: {
            const { formName, fieldName, status } = action.payload;
            return setEasyFormValue(state, formName, EEasyFormField.Statuses, fieldName, status);
        }
        case EEasyFormActionType.SetFieldErrors: {
            const { formName, fieldName, errors } = action.payload;
            return setEasyFormValue(state, formName, EEasyFormField.FieldErrors, fieldName, errors);
        }
        case EEasyFormActionType.ClearFieldErrors: {
            const { formName, fieldName } = action.payload;
            if (!((_b = state[formName]) === null || _b === void 0 ? void 0 : _b[EEasyFormField.FieldErrors])) {
                return state;
            }
            return setEasyFormValue(state, formName, EEasyFormField.FieldErrors, fieldName, null);
        }
        case EEasyFormActionType.InitiateForm: {
            const { formName, initialValues } = action.payload;
            const isEqualInitials = deepEqual(initialValues, (_c = state === null || state === void 0 ? void 0 : state[formName]) === null || _c === void 0 ? void 0 : _c[EEasyFormField.Initials]);
            if (isEqualInitials) {
                return state;
            }
            return Object.assign(Object.assign({}, state), { [formName]: Object.assign(Object.assign({}, (state[formName] || {})), { [EEasyFormField.Initials]: initialValues, [EEasyFormField.Statuses]: Object.keys(initialValues).reduce((acc, cur) => {
                        acc[cur] = EEasyFormFieldStatus.Pristine;
                        return acc;
                    }, {}) }) });
        }
        case EEasyFormActionType.SetFormErrors: {
            const { formName, errors } = action.payload;
            return Object.assign(Object.assign({}, state), { [formName]: Object.assign(Object.assign({}, (state[formName] || {})), { [EEasyFormField.FormErrors]: errors }) });
        }
        case EEasyFormActionType.DropForm: {
            const { formName } = action.payload;
            if (!state[formName]) {
                return state;
            }
            const { [EEasyFormField.Initials]: initials, [EEasyFormField.Statuses]: statuses } = state[formName];
            return Object.assign(Object.assign({}, state), { [formName]: {
                    [EEasyFormField.Initials]: initials,
                    [EEasyFormField.Statuses]: Object.keys(statuses).reduce((acc, cur) => {
                        acc[cur] = EEasyFormFieldStatus.Pristine;
                        return acc;
                    }, {}),
                } });
        }
        default:
            return state;
    }
};
//# sourceMappingURL=reducer.js.map