import { createSelector } from 'reselect';
import { EEasyFormField, EEasyFormFieldStatus } from './enums';
export const getForms = (state) => state.forms;
export const createGetForm = (formName) => (createSelector(getForms, (forms) => forms === null || forms === void 0 ? void 0 : forms[formName]));
export const createGetCommonFormErrors = (formName) => createSelector(createGetForm(formName), (form) => form === null || form === void 0 ? void 0 : form[EEasyFormField.FormErrors]);
export const createGetFormAllFieldsErrors = (formName) => createSelector(createGetForm(formName), (form) => form === null || form === void 0 ? void 0 : form[EEasyFormField.FieldErrors]);
export const createGetFormErrors = (formName) => createSelector(createGetCommonFormErrors(formName), createGetFormAllFieldsErrors(formName), (formErrors, fieldErrors) => {
    if (!fieldErrors && !formErrors) {
        return;
    }
    return Object.assign(Object.assign({}, (fieldErrors !== null && fieldErrors !== void 0 ? fieldErrors : {})), { [EEasyFormField.FormErrors]: formErrors });
});
export const createGetIsFormValid = (formName) => createSelector(createGetFormErrors(formName), (formErrors) => {
    if (!formErrors) {
        return true;
    }
    for (const key in formErrors) {
        if (Array.isArray(formErrors[key])) {
            return false;
        }
    }
    return true;
});
export const createGetFormValues = (formName) => (createSelector(createGetForm(formName), (form) => form === null || form === void 0 ? void 0 : form[EEasyFormField.Values]));
export const createGetFormInitialValues = (formName) => createSelector(createGetForm(formName), (form) => form === null || form === void 0 ? void 0 : form[EEasyFormField.Initials]);
export const createGetFormStatuses = (formName) => createSelector(createGetForm(formName), (form) => form === null || form === void 0 ? void 0 : form[EEasyFormField.Statuses]);
export const createGetIsFormPristine = (formName) => (createSelector(createGetFormStatuses(formName), (formStatuses) => {
    if (!formStatuses) {
        return true;
    }
    for (const key in formStatuses) {
        if (formStatuses[key] === EEasyFormFieldStatus.Dirty) {
            return false;
        }
    }
    return true;
}));
export const createGetFormFieldValue = (formName, fieldName) => createSelector(createGetFormValues(formName), (formValues) => formValues === null || formValues === void 0 ? void 0 : formValues[fieldName]);
export const createGetFormFieldInitialValue = (formName, fieldName) => createSelector(createGetFormInitialValues(formName), (formInitialValues) => formInitialValues === null || formInitialValues === void 0 ? void 0 : formInitialValues[fieldName]);
export const createGetFormFieldErrors = (formName, fieldName) => createSelector(createGetFormAllFieldsErrors(formName), (allFormFieldsErrors) => (allFormFieldsErrors === null || allFormFieldsErrors === void 0 ? void 0 : allFormFieldsErrors[fieldName]));
export const createGetIsFormFieldValid = (formName, fieldName) => createSelector(createGetFormFieldErrors(formName, fieldName), (formFieldErrors) => !formFieldErrors);
export const createGetFormFieldStatus = (formName, fieldName) => createSelector(createGetFormStatuses(formName), (formStatuses) => formStatuses === null || formStatuses === void 0 ? void 0 : formStatuses[fieldName]);
export const createGetIsFormFieldPristine = (formName, fieldName) => createSelector(createGetFormFieldStatus(formName, fieldName), (fieldStatus) => (!fieldStatus || fieldStatus === EEasyFormFieldStatus.Pristine));
export const createGetFormFieldSafetyValue = (formName, fieldName) => createSelector(createGetIsFormFieldPristine(formName, fieldName), createGetFormFieldInitialValue(formName, fieldName), createGetFormFieldValue(formName, fieldName), (isPristine, initialValue, value) => (isPristine ? value || initialValue : value));
export const createGetFormSafetyValues = (formName) => createSelector(createGetFormStatuses(formName), createGetFormInitialValues(formName), createGetFormValues(formName), (formStatuses, formInitials, formValues) => {
    if (!formStatuses || (!formInitials && !formValues)) {
        return;
    }
    const result = {};
    const setResultKeyValue = (key) => {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        result[key] = formStatuses[key] === EEasyFormFieldStatus.Pristine || !formStatuses[key]
            ? (formValues === null || formValues === void 0 ? void 0 : formValues[key]) || (formInitials === null || formInitials === void 0 ? void 0 : formInitials[key])
            : formValues === null || formValues === void 0 ? void 0 : formValues[key];
    };
    [formInitials, formValues]
        .filter(Boolean)
        .forEach((values) => {
        Object.keys(values).forEach(setResultKeyValue);
    });
    return result;
});
//# sourceMappingURL=selectors.js.map